#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-02-24 10:41:55
LastEditors: zmf96
LastEditTime: 2022-02-24 10:53:12
FilePath: /hotfinger/data/tmp.py
Description: 
"""

import json

new_data = []
with open("./hotfinger.json", "r", encoding="utf-8") as f:
    data = json.load(f)
    for item in data:
        print(item["name"])
        new_item = {
            "_id": item["_id"],
            "name": item["name"],
            "content": {"/": item["value"]},
            "tags": ["tide"],
        }
        new_data.append(new_item)

with open("../data/tmp.json", "w", encoding="utf-8") as f:
    json.dump(new_data, f, indent=4, ensure_ascii=False)



#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-03-02 08:15:49
LastEditors: zmf96
LastEditTime: 2022-03-02 09:06:48
FilePath: /hotfinger/finger_handle_tools/remove_err_finger.py
Description: 
"""
import json

ErrFingers = None
with open("../errfinger.json", "r") as f:
    ErrFingers = json.load(f)

# HotFingersV1 = None
# with open("../data/hotfinger_v1.json", "r") as f:
#     HotFingersV1 = json.load(f)
# HotFingersV2 = None
# with open("../data/hotfinger_v2.json", "r") as f:
#     HotFingersV2 = json.load(f)

hotfinger = []

# hotfinger.extend(HotFingersV1)
# hotfinger.extend(HotFingersV2)

with open("../data/hotfinger_v4.json", "r") as f:
    hotfinger = json.load(f)

print(len(hotfinger))

for errf in ErrFingers:
    print(errf)
    for k in errf.keys():
        for item in hotfinger:
            if k == item["name"]:
                hotfinger.remove(item)

print(len(hotfinger))

with open("../data/hotfinger_v5.json", "w", encoding="utf-8") as f:
    json.dump(hotfinger, f, indent=4, ensure_ascii=False)



#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-06-03 19:23:50
LastEditors: zmf96
LastEditTime: 2022-06-03 21:09:20
FilePath: /hotfinger/finger_handle_tools/handle_14.py
Description: 
"""
import json

import bson
import pymysql
from cp_common.log import logger

# # 打开数据库连接
# db = pymysql.connect("177.7.0.13","root","xsC4sdf43aX","15finger")
db = pymysql.connect(
    host="177.7.0.13", user="root", passwd="xsC4sdf43aX", db="15finger"
)

# 使用cursor()方法获取操作游标
cursor = db.cursor()


fingers = {}


def get_app():
    # SQL 查询语句
    sql = "SELECT * FROM api_app;"
    try:
        # 执行SQL语句
        cursor.execute(sql)
        # 获取所有记录列表
        results = cursor.fetchall()
        for row in results:
            # 打印结果
            # logger.info(row)
            get_app_finger(row[1], str(row[0]))
    except:
        logger.info("Error: unable to fetch data")


def get_app_finger(app_name, _id):
    try:
        sql = "select * from api_finger where app_id = %s" % _id
        cursor.execute(sql)
        results = cursor.fetchall()
        tmp = {
            "_id": {"$oid": str(bson.ObjectId())},
            "name": app_name,
            "content": get_content(results),
            "tags": ["14finger"],
        }
        fingers[app_name] = tmp

    except Exception as e:
        logger.info(app_name)
        logger.info(_id)
        logger.info(e)


def get_content(results):
    content = {}
    path_dict = {}
    for item in results:
        path = item[7]
        if not path:
            path = "/"
        if path in path_dict:
            path_dict[path].append(item)
        else:
            path_dict[path] = [item]
    for k, v in path_dict.items():
        ret = get_path(v)
        content[k] = ret
    try:
        return {"/": content["/"]}
    except Exception as e:
        logger.warning(e)
        return None


def get_path(results):
    ret = ""
    for item in results:
        if item[3] == "md5":
            ret = 'body_md5="' + item[2] + '"'
        else:
            _tmp = item[4] + '="' + item[2] + '"'
            if _tmp not in ret:
                if len(ret) > 1:
                    ret += " || "
                ret += _tmp
    return ret


def save2json(path, data):
    with open(path, "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4, ensure_ascii=False)


def getfinger():
    with open("../data/hotfinger_v5.json", "r") as f:
        hotfinger = json.load(f)
    return hotfinger


def main():
    get_app()
    save2json("./14finger.json", fingers)

    fingers_set = set(fingers)
    hotfinger_set = set()

    hotfinger = getfinger()
    for item in hotfinger:
        hotfinger_set.add(item["name"])

    difffinger = sorted(fingers_set - hotfinger_set)

    logger.info(difffinger)
    finger_v6 = []
    for item in difffinger:
        if fingers[item]["content"]:
            finger_v6.append(fingers[item])

    save2json("./finger_v6.json", finger_v6)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-02-24 07:52:34
LastEditors: zmf96
LastEditTime: 2022-03-30 19:27:37
FilePath: /hotfinger/hotfinger.py
Description: 
"""
import argparse
import hashlib
import json
import os
import random
import re
from concurrent.futures import ThreadPoolExecutor, as_completed

import requests
import urllib3
from bs4 import BeautifulSoup
from loguru import logger
from Wappalyzer import Wappalyzer, WebPage

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

# TODO: 获取目标网站基础信息
# TODO: 使用自研引擎识别网站类型
# TODO: 使用wappalyzer/webanalyzer识别网站类型

path_finger = {}
basedir = os.path.dirname(os.path.abspath(__file__))

ErrFingers = []

# re
retitle = re.compile(r'title="(.*)"')
reheader = re.compile(r'header="(.*)"')
rebody = re.compile(r'body="(.*)"')
rebanner = re.compile(r"banner=(.*)")
reother = re.compile(r'^(.*?)="(.*)"')
rebracket = re.compile(r"\((.*)\)")


class PathHotFinger:
    def __init__(self):
        self.url = ""
        self.title = ""
        self.headers = ""
        self.response = None
        self.response_size = 0
        self.body = ""
        self.body_md5 = ""
        self.status_code = 0
        self.fingers = []

    def run(self, url, fingers):
        self.url = url
        self._get_webpage()
        self._run(fingers)
        self._run_wappalyzer()
        self._run_webanalyzer()
        self.fingers = list(set(self.fingers))

    def _get_webpage(self):
        try:
            resp = requests.get(
                self.url,
                headers=self.requests_headers(),
                verify=False,
                timeout=10,
                allow_redirects=True,
            )
            self.response = resp
            _bs = BeautifulSoup(resp.text, "lxml")  # 创建BeautifulSoup对象
            self.title = _bs.title.string if _bs.title else ""
            self.headers = str(resp.headers)
            self.response = resp
            self.response_size = len(resp.text)
            self.body = resp.text
            self.body_md5 = self._get_md5(self.body)
            self.status_code = resp.status_code
        except Exception as e:
            logger.warning(e)

    def _check_rule(self, rule, name):
        try:
            if 'title="' in rule:
                title_value = re.findall(retitle, rule)
                if title_value[0].lower().strip() in self.title.lower():
                    return True
            elif 'body="' in rule:
                body_value = re.findall(rebody, rule)
                # st = rule.strip('() ')[6:-1]
                # if body_value[0] != st:
                #     logger.info(rule)
                #     logger.warning(body_value)
                #     logger.warning(st)
                if body_value[0] in self.body:
                    return True
            elif "header=" in rule:
                if re.findall(reheader, rule)[0] in self.headers:
                    return True
            elif "banner=" in rule:
                if re.findall(rebanner, rule)[0] in self.body:
                    return True
            # TODO: body_md5
            else:
                # logger.warning(rule)
                # logger.info(re.findall(reother,rule))
                if re.findall(reother, rule)[0][1] in self.headers:
                    return True
                return False

        except Exception as e:
            logger.warning(e)
            logger.debug(name)
            logger.debug(rule)
            ErrFingers.append({name: rule})
            return 0

    def _handle(self, name, content):
        # 满足一个条件即可的情况 1||2||3
        if (
            "||" in content
            and "&&" not in content
            and ("(" not in content or ")" not in content)
        ):
            for rule in content.split("||"):
                cr_ret = self._check_rule(rule, name)
                if cr_ret == True:
                    self.fingers.append(name)
                    break
                elif cr_ret == 0:
                    ErrFingers.append({name: content})
        # 只有一个条件的情况  1
        elif "||" not in content and "&&" not in content and "(" not in content:
            if self._check_rule(content, name):
                self.fingers.append(name)
        # 需要同时满足条件的情况 1&&2&&3
        elif "&&" in content and "||" not in content and "(" not in content:
            num = 0
            for rule in content.split("&&"):
                cr_ret = self._check_rule(rule, name)
                if cr_ret == True:
                    num += 1
                elif cr_ret == 0:
                    ErrFingers.append({name: content})
            if num == len(content.split("&&")):
                self.fingers.append(name)
        else:
            # 与条件下存在并条件: 1||2||(3&&4)
            if "&&" in re.findall(rebracket, content)[0]:
                for rule in content.split("||"):
                    if "&&" in rule:
                        num = 0
                        for _rule in rule.split("&&"):
                            cr_ret = self._check_rule(_rule, name)
                            if cr_ret == True:
                                num += 1
                            elif cr_ret == 0:
                                ErrFingers.append({name: content})
                        if num == len(rule.split("&&")):
                            self.fingers.append(name)
                            break
                    else:
                        cr_ret = self._check_rule(rule, name)
                        if cr_ret == True:
                            self.fingers.append(name)
                            break
                        elif cr_ret == 0:
                            ErrFingers.append({name: content})
            else:
                # 并条件下存在与条件： 1&&2&&(3||4)
                for rule in content.split("&&"):
                    num = 0
                    if "||" in rule:
                        for _rule in rule.split("||"):
                            cr_ret = self._check_rule(_rule, name)
                            if cr_ret == True:
                                num += 1
                            elif cr_ret == 0:
                                ErrFingers.append({name: content})
                    else:
                        cr_ret = self._check_rule(rule, name)
                        if cr_ret == True:
                            num += 1
                        elif cr_ret == 0:
                            ErrFingers.append({name: content})
                if num == len(content.split("&&")):
                    self.fingers.append(name)

    def _run(self, fingers):
        for item in fingers:
            try:
                for name, finger in item.items():
                    self._handle(name, finger)
            except Exception as e:
                logger.info(item)
                logger.info(finger)
                ErrFingers.append(item)
                logger.warning(e)

    def _run_wappalyzer(self):
        webpage = WebPage.new_from_response(self.response)
        wappalyzer = Wappalyzer.latest()
        ret = wappalyzer.analyze(webpage)
        self.fingers.extend(ret)

    def _run_webanalyzer(self):
        pass

    @staticmethod
    def requests_headers():
        user_agent = [
            "Mozilla/5.0 (Windows; U; Win98; en-US; rv:1.8.1) Gecko/20061010 Firefox/2.0",
            "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.6 Safari/532.0",
            "Mozilla/5.0 (Windows; U; Windows NT 5.1 ; x64; en-US; rv:1.9.1b2pre) Gecko/20081026 Firefox/3.1b2pre",
            "Opera/10.60 (Windows NT 5.1; U; zh-cn) Presto/2.6.30 Version/10.60",
            "Opera/8.01 (J2ME/MIDP; Opera Mini/2.0.4062; en; U; ssr)",
            "Mozilla/5.0 (Windows; U; Windows NT 5.1; ; rv:1.9.0.14) Gecko/2009082707 Firefox/3.0.14",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.106 Safari/537.36",
            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36",
            "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr; rv:1.9.2.4) Gecko/20100523 Firefox/3.6.4 ( .NET CLR 3.5.30729)",
            "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16",
            "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        ]
        UA = random.choice(user_agent)
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
            "User-Agent": UA,
            "Upgrade-Insecure-Requests": "1",
            "Connection": "keep-alive",
            "Cache-Control": "max-age=0",
            "Accept-Encoding": "gzip, deflate, sdch",
            "Accept-Language": "zh-CN,zh;q=0.8",
            "Referer": "http://www.baidu.com/",
        }
        return headers

    @staticmethod
    def requests_proxies():
        proxies = {"http": "", "https": ""}  # 127.0.0.1:1080
        return proxies

    @staticmethod
    def _get_md5(text):
        m = hashlib.md5()
        m.update(text.encode("utf-8"))
        return m.hexdigest()


class HotFinger:
    def __init__(self):
        self.url = ""
        self.fingers = []
        self.pages = dict()

    def run(self, url):
        self.url = url
        try:
            path_hot_finger = PathHotFinger()
            for path, fingers in path_finger.items():
                path_hot_finger.run(url + path, fingers)
                self.fingers.extend(path_hot_finger.fingers)
                self.pages[path] = path_hot_finger
        except Exception as e:
            print(e)


def init(fingers=[]):
    if fingers == []:
        with open(
            os.path.join(basedir, "data", "hotfinger_v5.json"), "r", encoding="utf-8"
        ) as f:
            fingers = json.load(f)
    logger.info("指纹数量" + str(len(fingers)))
    global path_finger
    path_finger = {}
    for item in fingers:
        for path, finger in item["content"].items():
            if path in path_finger.keys():
                path_finger[path].append({item["name"]: finger})
            else:
                path_finger[path] = [{item["name"]: finger}]


def worker(url):
    hf = HotFinger()
    hf.run(url)
    return hf


def save2json(path, data):
    with open(path, "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4, ensure_ascii=False)


def async_run(urls: list) -> list:
    with ThreadPoolExecutor(max_workers=5) as executor:
        future_to_url = {executor.submit(worker, url): url for url in urls}
        for future in as_completed(future_to_url):
            logger.info(future_to_url[future])
            logger.info(future.result().fingers)
    # logger.info(ErrFingers)



#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-02-24 08:26:18
LastEditors: zmf96
LastEditTime: 2022-03-30 19:02:35
FilePath: /hotfinger/__main__.py
Description: 
"""
import argparse
import json

from loguru import logger

from hotfinger import ErrFingers, async_run, init

if __name__ == "__main__":
    logger.debug("hotfinger  version 0.1")
    parser = argparse.ArgumentParser(description="hotfinger v0.1.0")
    parser.add_argument("-v", "--version", action="version", version="0.1.0")
    parser.add_argument(
        "-d",
        "--domain",
        type=str,
        help="目标域名,eg: https://www.baidu.com,https://bing.com",
        required=True,
    )
    args = parser.parse_args()
    logger.info("开始加载指纹库")
    init()
    urls = args.domain.split(",")
    print(urls)
    async_run(urls)
    with open("errfinger.json", "w", encoding="utf-8") as f:
        json.dump(ErrFingers, f, indent=4, ensure_ascii=False)



import json

from cp_common.log import logger

hotfinger = []


hotfinger_set = set()
scj_set = set()

with open("../data/hotfinger_v1.json", "r") as f:
    hotfinger = json.load(f)
    for item in hotfinger:
        hotfinger_set.add(item["name"].lower())


def remove_null(data):
    new_data = []
    for item in data:
        new_item = {}
        for k, v in item.items():
            if v != "":
                new_item[k] = v
        new_data.append(new_item)
        scj_set.add(item["name"].lower())
    return new_data


def diff_add(data):
    s = scj_set.difference(hotfinger_set)
    logger.info(s)
    logger.info(len(s))
    new_data = []
    for item in data:
        if item["name"].lower() in s:
            new_data.append(item)
    return new_data


def no_regex(data):
    new_data = []
    for item in data:
        new_item = {}
        title_content = None
        header_content = None
        html_content = None

        if "regex" not in item:
            content = ""
            if "title" in item:
                title_content = 'title="' + item["title"].strip() + '"'
                content = title_content
            if "headers" in item:
                if len(content) > 0:
                    content = content + " || "
                header_content = 'header="' + item["headers"].strip() + '"'
                content = content + header_content

            if "html" in item:
                if len(content) > 0:
                    content = content + " || "
                html_content = 'body="' + item["html"].strip() + '"'
                content = content + html_content

            new_item = {
                "_id": {"$oid": item["id"]},
                "name": item["name"],
                "content": {"/": content},
                "tags": ["scj"],
            }
            new_data.append(new_item)
        else:
            # TODO: 进一步处理正则
            new_item = {
                "_id": {"$oid": item["id"]},
                "name": item["name"],
                "content": {"/": "body=" + item["regex"].strip() + '"'},
                "tags": ["scj"],
            }
            new_data.append(new_item)
    return new_data


def yes_regex(data):
    pass


def main():
    with open("scj.json", "r", encoding="utf-8") as fp:
        data = json.load(fp)
        # logger.info(data["data"])
        data = remove_null(data["data"])

    with open("scj_v2.json", "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4, ensure_ascii=False)

    data = diff_add(data)

    with open("scj_v3.json", "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4, ensure_ascii=False)

    data = no_regex(data)
    logger.info(len(data))
    with open("scj_v4.json", "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4, ensure_ascii=False)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-03-02 17:34:50
LastEditors: zmf96
LastEditTime: 2022-05-25 21:20:51
FilePath: /hotfinger/finger_handle_tools/hotfinge2csv.py
Description: 
"""
import csv
import json

hotfinger = []

with open("../data/hotfinger_v3.json", "r") as f:
    hotfinger = json.load(f)
with open("../data/hotfinger_v3.csv", "w") as f:
    writer = csv.writer(f, delimiter=",")
    keys = ["id", "name", "tags"]
    writer.writerow(keys)
    for item in hotfinger:
        writer.writerow([item["_id"]["$oid"], item["name"], ",".join(item["tags"])])



#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-05-31 21:57:44
LastEditors: zmf96
LastEditTime: 2022-05-31 22:46:57
FilePath: /hotfinger/finger_handle_tools/handle_tidefinger.py
Description: 
"""

import json
import sqlite3

import bson
from cp_common.log import logger


def cmsfinger2json():
    conn = sqlite3.connect("./cms_finger.db")
    c = conn.cursor()

    cursor = c.execute("select * from tide;")
    tidefinger = {}
    for row in cursor:
        # logger.info(row)
        tidefinger[row[1]] = row
    conn.close()
    with open("tide2.json", "w", encoding="utf-8") as f:
        json.dump(tidefinger, f, indent=4, ensure_ascii=False)
    return tidefinger


def getfinger():
    with open("../data/hotfinger_v3.json", "r") as f:
        hotfinger = json.load(f)
    return hotfinger


def save2json(path, data):
    with open(path, "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4, ensure_ascii=False)


def main():

    tidefinger = cmsfinger2json()

    logger.info(len(tidefinger))

    hotfinger = getfinger()
    logger.info(len(hotfinger))

    hotfinger_set = set()
    tide_set = set(tidefinger)

    for item in hotfinger:
        hotfinger_set.add(item["name"])

    difffinger = sorted(tide_set - hotfinger_set)
    tide_set = sorted(tide_set)
    hotfinger_set = sorted(hotfinger_set)

    logger.info(difffinger)

    save2json("./tide_set.json", tide_set)
    save2json("./hotfinger_set.json", hotfinger_set)
    save2json("./diffinger.json", difffinger)

    finger_v4 = []

    for item in difffinger:
        logger.info(item)
        finger_v4.append(
            {
                "_id": {"$oid": str(bson.ObjectId())},
                "name": tidefinger[item][1],
                "content": {"/": tidefinger[item][2]},
                "tags": ["tide2"],
            }
        )
    save2json("./finger_v4.json", finger_v4)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hotfinger', 'hotfinger.finger_handle_tools']

package_data = \
{'': ['*'], 'hotfinger': ['data/*']}

install_requires = \
['bs4>=0.0.1,<0.0.2',
 'loguru>=0.6.0,<0.7.0',
 'lxml>=4.8.0,<5.0.0',
 'python-Wappalyzer>=0.3.1,<0.4.0',
 'webanalyzer>=2019.8.22,<2020.0.0']

setup_kwargs = {
    'name': 'hotfinger',
    'version': '0.1.4.2',
    'description': '',
    'long_description': None,
    'author': 'zmf96',
    'author_email': 'zmf96@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""
Author: zmf963 zmf96@qq.com
Date: 2022-05-25 21:54:13
LastEditors: zmf963 zmf96@qq.com
LastEditTime: 2022-06-02 13:57:40
FilePath: /hotfinger/hotfinger/__init__.py
Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
"""
#!/usr/bin/env python
# coding=utf-8

"""
Version: 0.1
Autor: zmf96
Email: zmf96@qq.com
Date: 2022-02-24 15:51:03
LastEditors: zmf96
LastEditTime: 2022-03-30 19:44:31
FilePath: /hotfinger/__init__.py
Description: 
"""

from cp_common.base import Plugin
from cp_common.utils import get_host_port_from_url

from .hotfinger import init, worker

__version__ = "0.1.4.2"

__all__ = ["PluginClass"]


class PluginClass(Plugin):
    usage = [
        {
            "name": "url_list",
            "type": "List[str]",
            "usage": 'Url列表，eg: ["http://baidu.com","http://bing.com"]',
        }
    ]
    plugin_name = "hotfinger"
    author = "example"
    version = __version__

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def run(self):
        self.logger.info(self.kwargs)
        init()
        url_list = self.kwargs.get("url_list")
        for url in url_list:
            finger = worker(url)
            print(finger.fingers)
            host, port = get_host_port_from_url(url)
            hostinfo = host + ":" + port
            self.results.append(
                {
                    "port_info": {
                        "hostinfo": hostinfo,
                        "host": host,
                        "port": port,
                        "products": finger.fingers,
                    }
                }
            )

    @staticmethod
    def vaildator_args(kwargs):
        if "url_list" in kwargs and isinstance(kwargs["url_list"], list):
            return True
        return False
